@@ -42,7 +42,7 @@ typedef struct {
 static GPtrArray *foreign_structs = NULL;
 
 static void
-init_foreign_structs ()
+init_foreign_structs (void)
 {
     foreign_structs = g_ptr_array_new ();
 }
