@@ -38,81 +38,6 @@ gboolean _pygi_marshal_from_py_void        (PyGIInvokeState   *state,
                                             PyGIArgCache      *arg_cache,
                                             PyObject          *py_arg,
                                             GIArgument        *arg);
-gboolean _pygi_marshal_from_py_boolean     (PyGIInvokeState   *state,
-                                            PyGICallableCache *callable_cache,
-                                            PyGIArgCache      *arg_cache,
-                                            PyObject          *py_arg,
-                                            GIArgument        *arg);
-gboolean _pygi_marshal_from_py_int8        (PyGIInvokeState   *state,
-                                            PyGICallableCache *callable_cache,
-                                            PyGIArgCache      *arg_cache,
-                                            PyObject          *py_arg,
-                                            GIArgument        *arg);
-gboolean _pygi_marshal_from_py_uint8       (PyGIInvokeState   *state,
-                                            PyGICallableCache *callable_cache,
-                                            PyGIArgCache      *arg_cache,
-                                            PyObject          *py_arg,
-                                            GIArgument        *arg);
-gboolean _pygi_marshal_from_py_int16       (PyGIInvokeState   *state,
-                                            PyGICallableCache *callable_cache,
-                                            PyGIArgCache      *arg_cache,
-                                            PyObject          *py_arg,
-                                            GIArgument        *arg);
-gboolean _pygi_marshal_from_py_uint16      (PyGIInvokeState   *state,
-                                            PyGICallableCache *callable_cache,
-                                            PyGIArgCache      *arg_cache,
-                                            PyObject          *py_arg,
-                                            GIArgument        *arg);
-gboolean _pygi_marshal_from_py_int32       (PyGIInvokeState   *state,
-                                            PyGICallableCache *callable_cache,
-                                            PyGIArgCache      *arg_cache,
-                                            PyObject          *py_arg,
-                                            GIArgument        *arg);
-gboolean _pygi_marshal_from_py_uint32      (PyGIInvokeState   *state,
-                                            PyGICallableCache *callable_cache,
-                                            PyGIArgCache      *arg_cache,
-                                            PyObject          *py_arg,
-                                            GIArgument        *arg);
-gboolean _pygi_marshal_from_py_int64       (PyGIInvokeState   *state,
-                                            PyGICallableCache *callable_cache,
-                                            PyGIArgCache      *arg_cache,
-                                            PyObject          *py_arg,
-                                            GIArgument        *arg);
-gboolean _pygi_marshal_from_py_uint64      (PyGIInvokeState   *state,
-                                            PyGICallableCache *callable_cache,
-                                            PyGIArgCache      *arg_cache,
-                                            PyObject          *py_arg,
-                                            GIArgument        *arg);
-gboolean _pygi_marshal_from_py_float       (PyGIInvokeState   *state,
-                                            PyGICallableCache *callable_cache,
-                                            PyGIArgCache      *arg_cache,
-                                            PyObject          *py_arg,
-                                            GIArgument        *arg);
-gboolean _pygi_marshal_from_py_double      (PyGIInvokeState   *state,
-                                            PyGICallableCache *callable_cache,
-                                            PyGIArgCache      *arg_cache,
-                                            PyObject          *py_arg,
-                                            GIArgument        *arg);
-gboolean _pygi_marshal_from_py_unichar     (PyGIInvokeState   *state,
-                                            PyGICallableCache *callable_cache,
-                                            PyGIArgCache      *arg_cache,
-                                            PyObject          *py_arg,
-                                            GIArgument        *arg);
-gboolean _pygi_marshal_from_py_gtype       (PyGIInvokeState   *state,
-                                            PyGICallableCache *callable_cache,
-                                            PyGIArgCache      *arg_cache,
-                                            PyObject          *py_arg,
-                                            GIArgument        *arg);
-gboolean _pygi_marshal_from_py_utf8        (PyGIInvokeState   *state,
-                                            PyGICallableCache *callable_cache,
-                                            PyGIArgCache      *arg_cache,
-                                            PyObject          *py_arg,
-                                            GIArgument        *arg);
-gboolean _pygi_marshal_from_py_filename    (PyGIInvokeState   *state,
-                                            PyGICallableCache *callable_cache,
-                                            PyGIArgCache      *arg_cache,
-                                            PyObject          *py_arg,
-                                            GIArgument        *arg);
 gboolean _pygi_marshal_from_py_array       (PyGIInvokeState   *state,
                                             PyGICallableCache *callable_cache,
                                             PyGIArgCache      *arg_cache,
@@ -153,11 +78,11 @@ gboolean _pygi_marshal_from_py_interface_flags    (PyGIInvokeState   *state,
                                                    PyGIArgCache      *arg_cache,
                                                    PyObject          *py_arg,
                                                    GIArgument        *arg);
-gboolean _pygi_marshal_from_py_interface_struct   (PyGIInvokeState   *state,
-                                                   PyGICallableCache *callable_cache,
-                                                   PyGIArgCache      *arg_cache,
-                                                   PyObject          *py_arg,
-                                                   GIArgument        *arg);
+gboolean _pygi_marshal_from_py_interface_struct_cache_adapter   (PyGIInvokeState   *state,
+                                                                 PyGICallableCache *callable_cache,
+                                                                 PyGIArgCache      *arg_cache,
+                                                                 PyObject          *py_arg,
+                                                                 GIArgument        *arg);
 gboolean _pygi_marshal_from_py_interface_interface(PyGIInvokeState   *state,
                                                    PyGICallableCache *callable_cache,
                                                    PyGIArgCache      *arg_cache,
@@ -185,27 +110,39 @@ gboolean _pygi_marshal_from_py_interface_instance (PyGIInvokeState   *state,
                                                    GIArgument        *arg);
 
 /* Simplified marshalers shared between vfunc/closure and direct function calls. */
+gboolean _pygi_marshal_from_py_basic_type (PyObject   *object,   /* in */
+                                           GIArgument *arg,      /* out */
+                                           GITypeTag   type_tag,
+                                           GITransfer  transfer);
+gboolean _pygi_marshal_from_py_basic_type_cache_adapter  (PyGIInvokeState   *state,
+                                                          PyGICallableCache *callable_cache,
+                                                          PyGIArgCache      *arg_cache,
+                                                          PyObject          *py_arg,
+                                                          GIArgument        *arg);
 
-gboolean pygi_marshal_from_py_gobject (PyObject *py_arg, /*in*/
+gboolean _pygi_marshal_from_py_gobject (PyObject *py_arg, /*in*/
+                                        GIArgument *arg,  /*out*/
+                                        GITransfer transfer);
+gboolean _pygi_marshal_from_py_gobject_out_arg (PyObject *py_arg, /*in*/
+                                                GIArgument *arg,  /*out*/
+                                                GITransfer transfer);
+
+gboolean _pygi_marshal_from_py_gvalue (PyObject *py_arg, /*in*/
                                        GIArgument *arg,  /*out*/
-                                       GITransfer transfer);
+                                       GITransfer transfer,
+                                       gboolean is_allocated);
 
-gboolean pygi_marshal_from_py_gvalue (PyObject *py_arg, /*in*/
-                                      GIArgument *arg,  /*out*/
-                                      GITransfer transfer,
-                                      gboolean is_allocated);
+gboolean _pygi_marshal_from_py_gclosure(PyObject *py_arg, /*in*/
+                                        GIArgument *arg); /*out*/
 
-gboolean pygi_marshal_from_py_gclosure(PyObject *py_arg, /*in*/
-                                       GIArgument *arg); /*out*/
-
-gboolean pygi_marshal_from_py_interface_struct (PyObject *py_arg,
-                                                GIArgument *arg,
-                                                const gchar *arg_name,
-                                                GIBaseInfo *interface_info,
-                                                GITypeInfo *type_info,
-                                                GType g_type,
-                                                PyObject *py_type,
-                                                GITransfer transfer,
+gboolean _pygi_marshal_from_py_interface_struct (PyObject *py_arg,
+                                                 GIArgument *arg,
+                                                 const gchar *arg_name,
+                                                 GIBaseInfo *interface_info,
+                                                 GITypeInfo *type_info,
+                                                 GType g_type,
+                                                 PyObject *py_type,
+                                                 GITransfer transfer,
                                                 gboolean is_allocated,
                                                 gboolean is_foreign);
 
