@@ -32,6 +32,8 @@
 #include <girepository.h>
 #include "pygi-cache.h"
 
+extern PyObject *PyGIDeprecationWarning;
+
 typedef struct {
     PyObject_HEAD
     GIRepository *repository;
@@ -44,6 +46,19 @@ typedef struct {
     PyGICallableCache *cache;
 } PyGIBaseInfo;
 
+typedef struct {
+    PyGIBaseInfo base;
+
+    /* Reference the unbound version of this struct.
+     * We use this for the actual call to invoke because it manages the cache.
+     */
+    struct PyGICallableInfo *py_unbound_info;
+
+    /* Holds bound argument for instance, class, and vfunc methods. */
+    PyObject *py_bound_arg;
+
+} PyGICallableInfo;
+
 typedef struct {
     PyGPointer base;
     gboolean free_on_dealloc;
